const { fs } = require('./../bootstrap/modules');
const commander = require('@flk/commander');
const packageManager = require('./../helpers/package-manager');

commander
    .command('i, install <packages...>', 'Install npm packages and add it to the app package.json file.')
    .example(`install ${process.env.CLI_NAME}-animator`)
    .option('--app', 'Define the application name that will contain the installed packages, default to baseApp value in config.json settings.')
    .then(installPackages);

async function installPackages(command) {
    let appName = command.options.app || 'common';

    let packages = command.packages;

    echo(`Installing ${cli.cyan(packages.join(', '))}`);

    await packageManager.install(packages);

    let appPackageFile = SRC_DIR + '/' + appName + '/package.json';

    let appPackageContent = fs.getJson(appPackageFile);

    if (!appPackageContent.packages) {
        appPackageContent.packages = [];
    }

    for (let $package of packages) {
        // this would be for versioning 
        // if ($package.includes('@')) {
        //     let {packageName, version} = $package.split('@');
        //     $package = packageName;
        // }

        appPackageContent.packages.push($package);
    }

    fs.putJson(appPackageFile, appPackageContent);

    die(cli.green('All Done.'));
}

module.exports = installPackages