const { fs } = require('./../bootstrap/modules');
const commander = require('@flk/commander');

const { zip } = require('zip-a-folder');

async function compressDistDirectory() {
    const zippedFile = ROOT + `/dist.zip`;
    await zip(ROOT + `/dist`, zippedFile);

    fs.move(zippedFile, ROOT + '/dist/dist.zip');
    echo(cli.greenBright('All applications have been built successfully.'));

    die();
}

async function upload() {
    const ftp = require("basic-ftp");

    const client = new ftp.Client()
    client.ftp.verbose = true
    try {
        await client.access({
            host: "crafted-internet.com",
            user: "crafted",
            password: "CI@2020",
            secure: false,
        });
        await client.uploadFrom(ROOT + '/bonga.json', "public_html/bonga2.json");
    }
    catch (err) {
        console.log(err)
    }
    client.close()
}

commander
    .command('produce [apps...]', 'Build applications for production..')
    .option('--base-url', 'Define base url, default from config.json file.')
    .option('--https', 'Allow only https requests by modifying htaccess file to force load https instead of http requests, default in config.json file')
    .option('--separated', 'When this flag is set, each application will be produced in a separate directory, otherwise all will be compiled to one directory.')
    .option('--without-htaccess', 'Do not create .htaccess file, use it when using nginx instead of apache.')
    .option('-wa, --without-assets', 'Do not copy any assets files as they maybe already created once, so we do not need to enlarge the final output.')
    .then(startProducing);

let htaccessFileContent = fs.get(FRAMEWORK_OUTPUTS_PATH + '/.htaccess');

let htaccessRulesPlaceholder = `
    RewriteCond %{REQUEST_FILENAME} !-d
    RewriteCond %{REQUEST_FILENAME} !-f
    RewriteRule ^appRule appName-locale.html [L, QSA]`;

const httpsOnly = `    
    # Force https
    RewriteCond %{HTTPS} off
    RewriteRule ^(.*)$ https://%{HTTP_HOST}%{REQUEST_URI} [L,R=301]`;

const prerenderText = (prerenderHandlerUrl, domainUrl, delay, bots) => `     
    # Prerender
    RewriteCond %{HTTP_USER_AGENT} .*(${bots}).* [NC]
    RewriteCond %{REQUEST_URI} !^(/public)
    RewriteRule (.*) ${prerenderHandlerUrl}?domain=${domainUrl}&path=/$1&delay=${delay} [L,QSA]`;

const sitemapText = (sitemapOptions) => {
    return `    
    # sitemap route
    RewriteRule ^${sitemapOptions.route}$ ${sitemapOptions.location} [L,NC]`;
};


// with 301 Redirect
// '    RewriteRule (.*) ${prerenderHandlerUrl}?domain=${domainUrl}&path=/$1&delay=${delay} [L,R=301]'

const DEFAULT_PRERENDER_OPTIONS = {
    mode: 'facade', // facade|external|internal
    internalPath: 'index.php',
    internalPath: 'prerender.php',
    delay: 2500,
    externalUrl: 'https://prerender.mentoor.io',
    bots: 'Google-Site-Verification|Googlebot|facebook|crawl|WhatsApp',
    staticDir: 'static-files',
    cache: 'redis',
};

const SITEMAP_DEFAULTS = {
    route: null,
    location: null,
};

const prerenderConfig = () => {
    let prerender = config.production.prerender;

    if (prerender === true) {
        prerender = DEFAULT_PRERENDER_OPTIONS;
    } else {
        prerender = Object.merge(DEFAULT_PRERENDER_OPTIONS, prerender);
    }

    return prerender;
}

function createIndexFile(command, apps) {
    return;
    let APPS_LIST = [];
    let routesList = {};
    let prerender = prerenderConfig();

    let internalFilePathLocation = FRAMEWORK_OUTPUTS_PATH + '/' + prerender.internalPath;

    let prerenderFile = fs.get(internalFilePathLocation);

    for (let i = 0; i < apps.length; i++) {
        let appName = apps[i];

        let app = config.apps[appName],
            appPath = app.path;

        // /admin => admin.en.html
        for (let localeCode of app.locales) {
            routesList[appPath + (appPath == '/' ? '' : '/') + localeCode] = appName + '-' + app.locale + '.html';
        }

        routesList[appPath] = appName + '-' + app.locale + '.html';
    }

    prerenderFile = prerenderFile.replace('const ENABLE_CACHE = true', `const ENABLE_CACHE = ${prerender.cache ? 'true' : 'false'}`);

    let appsArrayText = '[';

    for (let route in routesList) {
        let appFilePath = routesList[route];

        appsArrayText += `\n'${route}' => '${appFilePath}', \n`;
    }

    appsArrayText += ']';

    prerenderFile = prerenderFile.replace('APPS_LIST', appsArrayText);

    fs.put(ROOT + '/dist/' + prerender.internalPath, prerenderFile);
}

function createHtaccessFile(command, apps) {
    let htacessRulesList = [],
        mainApp = '',
        appsList = '',
        APPS_LIST = [];

    let outputHtaccessFileContent = htaccessFileContent.replace('main', mainApp);

    let httpsMode = typeof command.options.https != 'undefined' ? command.options.https : config.production.https;

    // HTTPS
    outputHtaccessFileContent = outputHtaccessFileContent.replace('#https', httpsMode ? httpsOnly : '');

    // Rules list
    let rulesList = [];

    // Prerender
    let prerender = config.production.prerender;
    if (prerender) {
        if (prerender === true) {
            prerender = DEFAULT_PRERENDER_OPTIONS;
        } else {
            prerender = Object.merge(DEFAULT_PRERENDER_OPTIONS, prerender);
        }

        if (prerender.mode === 'facade') {
            prerender.handlerUrl = prerender.internalPath;
        } else if (prerender.mode == 'external') {
            prerender.handlerUrl = prerender.externalUrl;
        }

        if (prerender) {
            outputHtaccessFileContent = outputHtaccessFileContent.replace('#Prerender', prerenderText(prerender.handlerUrl, config.production.baseUrl, prerender.delay, prerender.bots));
        }
    } else {
        outputHtaccessFileContent = outputHtaccessFileContent.replace('#Prerender', '');
    }

    let sitemap = Is.object(config.production.sitemap) ? config.production.sitemap : SITEMAP_DEFAULTS;

    if (sitemap.location && sitemap.route) {
        rulesList.push({
            comment: 'Sitemap',
            rule: `^${sitemap.route}$ ${sitemap.location}`,
        });
    }

    if (config.production.googleVerification) {
        rulesList.push({
            comment: 'Google Verification Code',
            rule: `^${config.production.googleVerification}$ ${config.production.googleVerification}`,
        });
    }

    if (config.production.rules) {
        for (let rule of config.production.rules) {
            let ruleOptions = {};

            if (rule.route && rule.location) {
                ruleOptions.rule = `^${rule.route}$ ${rule.location}`;
            } else if (rule.rule) {
                ruleOptions.rule = rule.rule;
            }

            if (rule.comment) {
                ruleOptions.comment = rule.comment;
            }

            rulesList.push(ruleOptions);
        }
    }

    let baseAppRuleOptions = {};

    for (let i = 0; i < apps.length; i++) {
        let appName = apps[i];

        let app = config.apps[appName],
            appPath = app.path.ltrim('/');

        let isBaseApp = i == apps.length - 1;

        for (let localeCode of app.locales) {
            let fileName = `${appName}-${localeCode}.html`;

            if (isBaseApp && localeCode == app.locale) {
                baseAppRuleOptions.comment = 'Base App';
                baseAppRuleOptions.rule = `^(.*)$ ${fileName}`;
                continue;
            }

            let appRule = localeCode == app.locale ? appPath : `${appPath}/${localeCode}`;

            let ruleOptions = {
                rule: `^${appRule} ${fileName}`,
            };

            rulesList.push(ruleOptions);
        }
    }

    rulesList.push(baseAppRuleOptions);

    let rulesText = '';

    for (let rule of rulesList) {
        if (rule.comment) {
            rulesText += `\n\t\t#${rule.comment.ltrim('#')}`;
        }

        rulesText += `\n\t\tRewriteRule ${rule.rule.ltrim('RewriteRule')} [L,QSA]`;
    }

    outputHtaccessFileContent = outputHtaccessFileContent.replace('#rulesList', rulesText);

    fs.put(ROOT + '/dist/.htaccess', outputHtaccessFileContent);

    if (prerender.mode == 'facade') {
        let prerenderFile = fs.get(FRAMEWORK_OUTPUTS_PATH + '/' + prerender.internalPath);

        prerenderFile = prerenderFile.replace('{staticDir}', prerender.staticDir).replace('RECACHE', prerender.recache ? 'true' : 'false')
            .replace('PRERENDER_URL', prerender.externalUrl);

        if (config.production.redis) {
            prerenderFile = prerenderFile.replace('INIT_REDIS', rediesText()).replaceAll('REDIS_CACHE', redisCache());
        }

        fs.put(ROOT + '/dist/' + prerender.internalPath, prerenderFile);
    }
}

async function startProducing(command) {
    if (!fs.exists(MAIN_CONFIG_PATH)) {
        die(cli.white(`Framework is not configured, please run ${cli.green(process.env.CLI_NAME + ' fresh-install')} first.`));
    }

    let config = fs.getJson(MAIN_CONFIG_PATH);
    // remove the old dist
    fs.remove(ROOT + '/dist');
    // create again
    fs.makeDirectory(ROOT + '/dist');

    let apps = !Is.empty(command.args.apps) ? command.args.apps : Object.keys(config.apps);

    global.echo.sameLine = console.log;

    for (let appName of apps) {
        require(FRAMEWORK_ROOT_PATH + '/bootstrap/app-config')(appName);

        if (command.options.baseUrl) {
            global.BASE_URL = command.options.baseUrl;
        }

        let { sassCompiler } = require(FRAMEWORK_ROOT_PATH + '/bootstrap');

        sassCompiler.setOptions({
            root: ROOT,
            srcDir: SRC_DIR,
            appDir: APP_DIR,
            appName: APP_NAME,
        });

        const buildApp = require('./../app-compiler');
        // const serviceWorker = require('./../build/builders/service-worker-handler');

        // // check if there's a service worker
        // if (serviceWorker.check(appName)) {
        //     serviceWorker.copy();
        // }
        // die();
        await buildApp();

        // sassCompiler.clear('rtl');
        // echo(cli)

        for (let direction of appConfig.cssDirections) {
            sassCompiler.clear(direction);
            echo(cli.hex('#DEADED').bold(`Compiling ${direction} CSS direction Files...`))
            await sassCompiler.compile(direction);
        }

        await require('./../production/compiler')(appName, command);
    }

    echo(cli.yellowBright('Generating htacess file...'));
    createHtaccessFile(command, apps);
    createIndexFile(command, apps);

    // zip the directory
    echo(cli.redBright('Compressing dist directory'));
    compressDistDirectory();

}

module.exports = startProducing;