function parseForLoopString(forStatement) {
    let forOptions = {};

    if (forStatement.includes(' of ')) {
        forOptions = parseForOf(forStatement);
    } else if (forStatement.includes(' in ')) {
        forOptions = parseForIn(forStatement);
    } else if (forStatement.repeatsOf(';') == 2) { // syntax: let i = 0; i < 6; i++
        forOptions = parseNormalFor(forStatement);
    }

    return forOptions;
}

/**
 * Expected syntax:
 * A: let item of this.items
 * B: let item, index of this.items
 */
function parseForOf(forStatement) {
    let index = 'i', value, keyValue, iteratorVariable, variableIdentifier;
    [keyValue, iteratorVariable] = forStatement.split(' of ').map(string => string.trim());

    if (keyValue.includes(',')) {
        [value, index] = keyValue.split(/\s*,\s*/);
    } else {
        value = keyValue;
    }

    [variableIdentifier, value] = value.split(/\s+/).map(string => string.trim());
    
    // if (value == 'i') {
    //     index = 'k';
    // }

    let statement = `${variableIdentifier} ${value} of ${iteratorVariable}`;

    return {
        statement,
        type: 'forof',
        indexVariable: index,
        generatedIndex: 'idx' + random(4),
    };
}

/**
 * Get the proepr key that will be used inside the loop elements
 * @param {*} forStatement 
 */

/**
 * Expected syntax: let i = 0; i < 6; i++
 */
function parseNormalFor(forStatement) {
    let index, initial, iteratorVariable, variableIdentifier, finalExpression;
    [initial, iteratorVariable, finalExpression] = forStatement.split(';');

    // replace multiple spaces into one
    [variableIdentifier, index] = initial.replace(/(\s)+/, ' ').split(/\s+/);

    return {
        type: 'numeric',
        indexVariable: index,
        statement: forStatement,
    };
}

/**
 * Expected syntax:
 * A: let key in this.object
 * B: let key, value in this.object
 */
function parseForIn(forStatement) {
    let index, value = 'value', keyValue, iteratorVariable, variableIdentifier;
    [keyValue, iteratorVariable] = forStatement.split(' in ').map(string => string.trim());

    if (keyValue.includes(',')) {
        [index, value] = keyValue.split(/\s*,\s*/);
    } else {
        index = keyValue;
    }

    [variableIdentifier, index] = index.split(/\s+/).map(string => string.trim());

    let statement = `${variableIdentifier} ${index} in ${iteratorVariable}`;

    return {
        statement,
        iteratorVariable,
        type: 'forin',
        variableIdentifier,
        valueVariable:value,
        indexVariable: index,
    };
}

module.exports = {
    parseForLoopString,
};