const $ = require('cheerio');
const { fs } = require('@flk/fs');
const events = require('@flk/events');

const { parseForLoopString } = require('./parse-for-loop');

// Error.stackTraceLimit = Infinity;

let root;

const domOptions = {
    withDomLvl1: true,
    normalizeWhitespace: false,
    xmlMode: true,
    decodeEntities: true
};

function pullAttr(el, attribute, $default = null) {
    let value = el[0].attribs[attribute] || $default;

    el.removeAttr(attribute);

    return value;
}

function handleAttributes(attributes) {
    if (!attributes) return {};

    for (let attribute in attributes) {
        let value = attributes[attribute];

        if (value === 'false') {
            value = false;
        }

        if (value === 'true') {
            value = true;
        }

        if (Is.string(value)) {
            value = value.trim();
        }

        if (value === '') {
            value = true;
        }

        if (Is.numeric(value)) {
            value = Number(value);
        }

        attributes[attribute] = value;
    }

    return attributes;
}

// will be used to collect current if|elseif|else statements 
// and will be cleared at the end of each if clause 
let states = [];

// will store all if states to update the tags list later
let ifStates = {};


function createElement(name, attrs = {}, ...children) {
    let jQUery = $.load(`<${name} />`, domOptions);

    let element = jQUery(name);

    element.attr(attrs);

    for (let child of children) {
        element.append(child);
    }

    return jQUery;
}

const parseChildren = (children, parentOptions = { key: '' }, parent) => {
    let list = [];

    if (!children) return list;

    try {
        for (let child of children) {
            let otherNonChildren = [];

            if (/^(input|img|br|hr|meta|iframe|link|embed|base|area|param|source|track)$/.test(child.name) && child.children.length > 0) {
                otherNonChildren = child.children;
                child.children = [];
            }

            let element = parse(child, Object.assign({}, parentOptions), parent);
            if (element) {
                // if (parentOptions && !Is.undefined(parentOptions.childrenStates) && element.childrenStates) {
                //     echo(element.childrenStates)
                //     parentOptions.childrenStates = (parentOptions.childrenStates || []).concat(element.childrenStates)                    
                // }

                list.push(element);
            }

            list = list.concat(parseChildren(otherNonChildren, parentOptions, parent));
        }
    } catch (error) {
        echo()
        echo(cli.redBright(`Error In: ${parser.htmlCompiler.viewName}`));
        die(error)
    }

    return list;
};

function parse(element, parentOptions = { key: '' }, parent) {
    let dataText = element.data;

    if (element.type == 'text') {
        dataText = element.data.trim();
        if (!dataText) return;
    }

    let $el = root(element),
        key = parentOptions.key;

    let object = {
        outerHTML: $.html($el),
        innerHTML: $el.html(),
        insideLoop: parentOptions.insideLoop || false,
        key: parentOptions.key,
        name: element.name,
        type: element.type,
        attributes: handleAttributes(element.attribs),
    };

    if (['if', 'elseif', 'else'].includes(element.name)) {
        if (key) {
            key += '_';
        }

        key += 'i' + random(4);

        object.key = key;

        parentOptions.key = key;

        // let state = '${this.key + (`st' + random(4);
        let state = 'st' + random(4);

        if (parentOptions.keyVariable) {
            state += '${' + parentOptions.keyVariable + '}';
        }

        // state += '`)}';

        let conditionAlias = 'cnd' + random(4);
        object.state = state;
        object.alias = conditionAlias;

        parentOptions.state = state;

        if (!Is.undefined(parentOptions.childrenStates)) {
            parentOptions.childrenStates.push(state);
        }

        parentOptions.childrenStates = [];

        let condition = pullAttr($el, 'condition', '');

        if (element.name == 'if') {
            states.push({
                state,
                conditionsList: [
                    {
                        state,
                        condition,
                        alias: conditionAlias,
                    }
                ],
                conditions: [
                    conditionAlias
                ]
            });
        } else if (element.name == 'elseif') {
            let lastState = Array.end(states);
            lastState.conditions.push(conditionAlias);
            lastState.conditionsList.push({
                state,
                condition,
                alias: conditionAlias,
            });
        } else if (element.name == 'else') {
            let lastState = Array.end(states);

            // Negate all if/elseif statements
            condition = `!(${lastState.conditions.join('||')})`;

            lastState.conditions.push(conditionAlias);

            lastState.conditionsList.push({
                state,
                condition,
                alias: conditionAlias,
            });
        }

        object.condition = condition;

        // terminate last state when this if statement is the last one in its
        // siblings
        if (['if', 'elseif'].includes(element.name)) {
            try {
                let lastStatement = true;

                let siblings = root(element).nextAll();

                for (let i = 0; i < siblings.length; i++) {
                    let item = siblings[i];

                    if (item.name == 'if') {
                        break;
                    }

                    if (item.name == 'elseif' || item.name == 'else') {
                        lastStatement = false;
                        break;
                    }
                }

                if (lastStatement === true) {
                    let lastState = states.pop();
                    events.trigger('end-of.if', lastState);
                }
            } catch (error) {
                echo()
                echo(cli.redBright(`Error In: ${parser.htmlCompiler.viewName}`));
                die(error)
            }
        } else if (element.name == 'else') {
            let lastState = states.pop();
            events.trigger('end-of.if', lastState);
        }
    }

    if (element.name == 'for') {
        if (key) {
            key += '_';
        }

        object.loop = pullAttr($el, 'loop');

        key += 'f' + random(4);

        object.forOptions = parseForLoopString(object.loop);

        // get the index from the statement to append it to the key
        let index = object.forOptions.indexVariable;

        object.loopKey = pullAttr($el, '*key', null);

        if (!object.loopKey || object.loopKey == 'null') {
            object.loopKey = index;
        }

        key += '${' + object.loopKey + '}';

        object.forKey = key;

        object.index = index;

        object.keyVariable = 'fk' + random(4);

        parentOptions.key = '${' + object.keyVariable + '}';

        parentOptions.insideLoop = true;
        parentOptions.index = index;
        parentOptions.keyVariable = object.keyVariable;

        object.key = parentOptions.key;
    }

    if (element.type == 'text' || element.type == 'comment') {
        object.data = dataText.trim();
    }

    object.children = parseChildren(element.children, parentOptions, parent);

    if (parentOptions.state) {
        object.state = parentOptions.state;
    }

    if (parentOptions.index) {
        object.index = parentOptions.index;
    }

    if (parentOptions.childrenStates) {
        object.childrenStates = parentOptions.childrenStates;
    }

    return object;
}

let piper = {
    pipelines: [],
    pipeline(pipeline) {
        this.pipelines.push(pipeline);
        return this;
    },
};

let parser = {
    parse() {
        let modifiedHtml = this.html;

        for (let pipeline of piper.pipelines) {
            modifiedHtml = pipeline.call(this, modifiedHtml, this.htmlCompiler, this.root);
        }

        this.rootElement = this.root('root');

        this.rootElement.html(modifiedHtml);

        states = [];

        ifStates = {};

        events.off('end-of.if').on('end-of.if', state => {
            // echo(state.state)
            ifStates[state.state] = state;
        });

        fs.put(__dirname + '/test.html', modifiedHtml)

        let tags = parseChildren(this.rootElement[0].children, { key: '' }, this.rootElement);

        checkStates(tags);

        // update all if tags with the its proper if states


        // die(fs.putJson(__dirname + '/test.json', tags));

        return tags;
    }
}

function checkStates(tags) {
    for (let tag of tags) {
        if (tag.name == 'if' && ifStates[tag.state]) {
            tag.statesList = ifStates[tag.state].conditionsList;

            // for performance wise, we'll delete the state
            // from the list as it won't be needed to any other states
            // delete ifStates[tag.states];
        }

        checkStates(tag.children);
    }
}

// let selfClosedRegex = /\<(input|img|br|hr|meta|iframe|link|embed|base|area|param|source|track)([^\/|\>]*)\>/gm;
let selfClosedRegex = /\<(input|img|br|hr|meta|iframe|link|embed|base|area|param|source|track)(>|.*?[^?]>)/gm;


/**
 * Adjust self closed tag in html 5 mode
 * as we need to convert for example <input > to <input />
 * @param   {string} html
 * @returns {string}
 */
function adjustSelfClosedTags(html) {
    return html.replace(selfClosedRegex, function (full, tagName, attributes) {
        return `<${tagName}${attributes} />`;
    });
}

function parseRoot(html, htmlCompiler) {
    // html = adjustSelfClosedTags(html);
    parser.root = root = createElement('root');
    parser.html = html;
    parser.htmlCompiler = htmlCompiler;

    return parser;
}


module.exports = {
    pullAttr,
    parseRoot,
    domOptions,
    createElement,
    piper,
}   