const Tag = require('./tag');

module.exports = class Component extends Tag {
    /**
     * {@inheritdoc}
     */
    type() {
        return 'component';
    }

    /**
     * {@inheritdoc}
     */
    asElement() {
        return this.className().toCamelCase();
    }

    /**
     * {@inheritdoc}
     */
    className() {
        return this.element.name + '-el';
    }

    /**
     * {@inheritDoc}
     */
    build() {
        let componentInfo = {},
        finalOutput;
        let skippedProperties = [this.variableName];
        
        if (this.contentToString && this.innerHTML()) {
            componentInfo.contentToString = '`' + this.innerHTML().replaceAll('`', '\\`') + '`';
            skippedProperties.push(componentInfo.contentToString);
        } else if (this.parseContent !== false) {
            // let parser = new Parser(this.innerHTML(), this.htmlFile),
            //     contentCompiler = parser.htmlCompiler;

            // contentCompiler.originalCompiler = this.htmlCompiler.originalCompiler;

            // contentCompiler.parentComponent = this;

            // let parsedComponentContent = contentCompiler.parse();

            let parentName = this.asParentName();

            if (this.specialAttr('extract-as')) {
                this.onBuild(_ => `let {${this.specialAttr('extract-as')}} = ${parentName}; \n`);
            }
            
            this.htmlCompiler.parentName = this.asParentName();

            this.htmlCompiler.parentComponent = this;
            // finalOutput = `function (${this.asParentName()}) {
            //     ${this.constructTag()}${this.parseChildren(this.htmlCompiler.parentComponent)}${this.terminate()}
            // }`;

            finalOutput = this.childrenOutput(this.parseChildren(this.htmlCompiler.parentComponent));
            
            this.htmlCompiler.parentComponent = null;
        }

        componentInfo.parent = this.htmlCompiler.originalCompiler.objectName;
        skippedProperties.push(componentInfo.parent);

        // if (this.parentComponent && this.parentComponent != this) {
        //     componentInfo.parentTop = this.parentComponent.asParentName();
        //     skippedProperties.push(componentInfo.parentTop);
        // }

        if (this.parentComponent) {
            componentInfo.parentTop = this.parentComponent.asParentName();
        }


        if (this.isInsideIfStatement()) {
            componentInfo.state = `\`${this.state()}\``;
        }

        if (this.isInsideForLoop()) {
            componentInfo.insideLoop = true;
            componentInfo.index = this.element.index;
        }

        if (this.isInsideForLoop() || this.isInsideIfStatement()) {
            // the unique id of the for loop is used because there might be nested loops
            // so we want to make sure that any component inside all of these loops have
            // a unique index
            // componentInfo.key = '`${this.key + (' + this.key() + ')}`';
            componentInfo.key = this.key();
            skippedProperties.push(componentInfo.key);
        }

        // let componentElementVariable = `el${random(5)}`;

        // let componentElement = `
        //     let ${componentElementVariable} = ${Tag.ELEMENT_OPEN}('${this.tagName()}')
        // `;

        // skippedProperties.push(componentElementVariable);

        // parsed += (componentElement);

        // componentInfo.parentNode = componentElementVariable;

        if (finalOutput) {
            componentInfo.content = finalOutput;
        }

        componentInfo.id = '`' + random(5) + (componentInfo.key ? String(componentInfo.key).trim('`') : '') + '`';

        let secondArgument = '';

        let finalAttributes = this.getFinalAttributes();

        if (! Is.empty(finalAttributes)) {
            componentInfo.props = finalAttributes;
        }

        let flat = object => {
            let propsList = '{';

            for (let property in object) {
                let value = object[property];
                if (Is.array(value)) {
                    propsList += '[';
                    for (let arrayValue of value) {
                        propsList += `${arrayValue},`;
                    }

                    propsList += ']';
                }
                else if (Is.object(value)) {
                    value = flat(value);
                }
                propsList += `'${property}': ${value},`;
            }

            propsList += '}';

            return propsList;
        };

        componentInfo.parentNode = this.asElement();

        if (!Is.empty(componentInfo)) {
            // componentInfo = flatten(componentInfo, {
            //     skip: skippedProperties,
            // });
            secondArgument = `, ${flat(componentInfo)}`;
        }

        let componentCode = `this._lc('${this.tagName()}'${secondArgument})`;

        let output = this.output(componentCode);

        // return output;

        return `elem('${this.tagName()}', {
            key: '${random(5)}',
        }, ${this.asElement()} => {
            ${output}
        });`;

        return `elem('span', {
            key: '${random(5)}',
            class: '${this.className()}',
        }, ${this.asElement()} => {
            ${output}
        });`;
    }

    /**
     * Collect passed custom events
     */
    collectCustomEvents() {
        let customEvents = {};
        for (let attribute in this.originalAttributes) {
            const addEvent = event => {
                let eventName = 'on' + event;
                customEvents[eventName] = `function(e) {let $el = this; ${this.originalAttributes[attribute]}}`;
            };
            if (attribute.startsWith('(') && attribute.endsWith(')')) {
                let event = attribute.ltrim('(').rtrim(')').toCamelCase();

                if (event.includes(',')) {
                    event.split(',').map(event => {
                        addEvent(event.trim());
                    })
                } else {
                    addEvent(event);
                }
            }

            // let functionArguments = this.collectArgumentsOf(this.originalAttributes[attribute]);

            // customEvents[eventName] = `function(${functionArguments}) {${this.originalAttributes[attribute]}}`;
        }

        return customEvents;
    }

    /**
     * Get the parameters inside the braces ()
     * 
     * i.e this.update(value1, value2) so we will return value1, value2
     */
    collectArgumentsOf(functionCode) {
        let fullArgumentsList = [];

        functionCode.split(';').forEach(string => {
            string.replace(/\((.*)\)/g, function (matched, argumentsList) {
                // split arguments by semi colon
                argumentsList.split(',').forEach(argument => {
                    argument = argument.trim();

                    if (!fullArgumentsList.includes(argument)) {
                        fullArgumentsList.push(argument);
                    }
                });
            });
        });

        return fullArgumentsList.join(',');
    }
}