class UsersReportsPage extends Table {
    /**
     * Constructor
     * Put your required dependencies in the constructor parameters list  
     */
    constructor(router, events, usersReportsService, bulkService) {
        super(usersReportsService);
        this.events = events;
        this.router = router;
        this.bulkService = bulkService;
        this.name = 'users-reports';
        this.title = trans('usersReports');

        this.bulkLoads = ['usersGroups', 'categories'];

        this.options = {
            permissionsKey: 'usersReports',
            singleName: null,
            table: {
                heading: 'usersReports',
                actions: false,
                columns: [{
                    name: 'id',
                    head: '#',
                    key: 'id',
                }, {
                    head: 'name',
                    key: 'name',
                }, {
                    head: 'group',
                    key: 'group.name',
                }, {
                    head: 'news',
                    key: 'totalNews',
                    default: 0,
                    formatter: 'number',
                }],
                filter: {
                    request: this.service.list.bind(this.service),
                    response: this.onFilterResponse.bind(this),
                    options: [{
                        type: 'dropdown',
                        text: trans('usersGroups'),
                        param: 'usersGroups[]',
                        multiple: true,
                        columnSize: 4,
                        items: () => {
                            return this.load('usersGroups');
                        },
                    }, {
                        type: 'dropdown',
                        text: trans('categories'),
                        param: 'categories[]',
                        multiple: true,
                        columnSize: 4,
                        items: () => {
                            return this.load('categories');
                        },
                    }, {
                        type: 'date',
                        text: trans('startDate'),
                        param: 'from',
                        columnSize: 4,
                    }, {
                        type: 'date',
                        text: trans('endDate'),
                        param: 'to',
                        columnSize: 4,
                    }],
                },
            },
        };
    }
}